/****************************************************************************************************

		IScenePreviewPart.h
		Copyright: (c) 2001 Eovia Corp. All rights reserved.

		Author:	Charles
		Date:	2/10/2003

****************************************************************************************************/

#ifndef __IScenePreviewPart__
#define __IScenePreviewPart__

#if CP_PRAGMA_ONCE
#pragma once
#endif

#include "APITypes.h"
#include "MCCompObj.h"


struct I3DShScene;


// ****************************************************************
//				definition of the GUIDs
//
MCDEFINE_GUID(IID_IScenePreviewPart, 0xf27d31c, 0x4443, 0x4e52, 0x96, 0x38, 0xf0, 0xa, 0x2f, 0x63, 0xd3, 0xcd);

/// \addtogroup ShellInterface
/// \{



// ****************************************************************************
//								IScenePreviewPart
// 
/// Interface for a generic scene preview.
struct IScenePreviewPart : public IMCUnknown
{
	enum ERenderingMode
	{
		kRaytracing,
		kZBuffer
	};

	// ****************************************************************************
	//								SetScene
	// 
	/// Sets the scene to be rendered
	/// 
	/// \param scene  Pointer to the scene to render
	/// 
	virtual void		MCCOMAPI SetScene(I3DShScene* scene)=0;

	// ****************************************************************************
	//								RenderScene
	// 
	/// Render the preview.
	///  
	virtual void		MCCOMAPI RenderScene(boolean synchronous=false)=0;

	// ****************************************************************************
	//								AbortRendering
	// 
	/// Abirts the current rendering
	/// 
	/// 
	virtual void	    MCCOMAPI AbortRendering()=0;

	// ****************************************************************************
	//								SetRenderingMode
	// 
	/// Sets the rendering mode used for the preview
	/// 
	/// \param renderingMode  The mode: Raytracing or ZBuffer.
	/// 
	virtual void		MCCOMAPI SetRenderingMode(ERenderingMode renderingMode)=0;

	// ****************************************************************************
	//								Invalidate
	// 
	/// Invalites the rendering
	/// 
	virtual void		MCCOMAPI ClearRender()=0;
};


/// \}

#endif
